#ifndef LIN_KERNIGHAN_H
#define LIN_KERNIGHAN_H

#include "tsp.h"

/**
 * Prepare linKernighan to be ready to run.
 * @param graph    TSP graph
 * @param nNeighs  number of neighbours for each node
 * @param neighs   neighbours for each node
 */
void initLinKernighan(Graph* graph, int nNeighbours, int** neighbours);


/**
 * Optimizes given route using Lin-Kernighan algorithm. Correspond to Algorithm
 * 2.4 in the article "Finding Tours in the TSP" (Applegate, Bixby, Chvatal, Cook)
 * @param r         route to optimize
 * @param callback  callback function, that is called after each improvement
 *                  if NULL, no function call is made
 */
void linKernighan(Route* route, void (*callback)(Route* route));

#endif

